!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!
! Random number generator ILFSR113  - integer version
!
! Following a suggestion of Pierre L'Ecuyer 1997
! "Tables of maximally equidistributed combined LFSR generators"
! see http://www.iro.umontreal.ca/~lecuyer/papers.html
!
! A call to ilfsr113() gives one random integer in the 
! interval (1,2147483647).
!
! Before using ilfsr113 call lfsrinit(seed) to initialize
! the generator by random integers produced by Park/Millers
! minimal standard LCG.
! Seed should be any positive integer.
! 
! FORTRAN version by Thomas Vojta, vojta@physik.tu-chemnitz.de
! 
! History:
!  04 Feb 1998    v0.9    first FORTRAN implementation
!  10 Feb 2000    v0.91   changed lfsrinit to allow for CONSTANT arguments
!  26 Apr 2011    v0.92   changed integer representation test in lfsrinit
!                         to avoid problems with binary constants                   
! 
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


      FUNCTION ilfsr113 ()
      implicit none
      integer,parameter         :: r8b= SELECTED_REAL_KIND(P=12,R=99) 
      integer,parameter         :: i4b= SELECTED_INT_KIND(8)

      integer(i4b) b,z1,z2,z3,z4,ilfsr113
      common /lfsrcom/z1,z2,z3,z4

      b  = ishft(ieor(ishft(z1,6),z1),-13)
      z1 = ieor(ishft(iand(z1,-2),18),b)

      b  = ishft(ieor(ishft(z2,2),z2),-27)
      z2 = ieor(ishft(iand(z2,-8),2),b)

      b  = ishft(ieor(ishft(z3,13),z3),-21)
      z3 = ieor(ishft(iand(z3,-16),7),b)

      b  = ishft(ieor(ishft(z4,3),z4),-12)
      z4 = ieor(ishft(iand(z4,-128),13),b)

      ilfsr113=ishft( ieor(ieor(ieor(z1,z2),z3),z4) , -1)

      return
      end


      SUBROUTINE lfsrinit(iinit)
      implicit none
      integer,parameter         :: r8b= SELECTED_REAL_KIND(P=12,R=99) 
      integer,parameter         :: i4b= SELECTED_INT_KIND(8)

      integer(i4b),parameter    :: im=2147483647
      integer(i4b),parameter    :: ia=16807
      integer(i4b),parameter    :: iq=127773
      integer(i4b),parameter    :: ir=2836

      integer(i4b) idum,iinit,c1
      integer(i4b) ilfsr113
      integer(i4b) k,z1,z2,z3,z4
      common /lfsrcom/z1,z2,z3,z4

      !!! Test integer representation !!!
      c1=-8
      c1=ishftc(c1,-3)
!     print *,c1
      if (c1.ne.536870911) then
         print *,'Nonstandard integer representation. Stoped.'
         stop
      endif

      idum=iinit
      if (idum.le.0) idum=1
      k=(idum)/IQ
      idum=IA*(idum-k*IQ)-IR*k
      if (idum.lt.0) idum = idum + IM
      if (idum.lt.2) then
         z1=idum+2 
      else 
         z1=idum
      endif
      k=(idum)/IQ
      idum=IA*(idum-k*IQ)-IR*k
      if (idum.lt.0) idum = idum + IM
      if (idum.lt.8) then 
         z2=idum+8 
      else 
         z2=idum
      endif
      k=(idum)/IQ
      idum=IA*(idum-k*IQ)-IR*k
      if (idum.lt.0) idum = idum + IM
      if (idum.lt.16) then
         z3=idum+16 
      else 
         z3=idum
      endif
      k=(idum)/IQ
      idum=IA*(idum-k*IQ)-IR*k
      if (idum.lt.0) idum = idum + IM
      if (idum.lt.128) then
         z4=idum+128 
      else 
         z4=idum
      endif

      idum=ilfsr113()
      
      return
      end


